# Released under the GNU Lesser General Public License, v2.1 or later
# Copyright (c) 2007 Phil Jones <interstar@gmail.com>

import re

# Some miscellaneous helper routines and objects.

def spaceUnder(s) :
    return re.sub('\s+','_',s)
    
def fromFile(fName, cDir = '') :
    f = open(cDir + fName)
    s = f.read()
    f.close()
    return s

def jFact(f, depth=0, cDir='', sfx = lambda x : x) :
    # make a join function with depth, cDir, sfx in the closure
    def g(node, wrap=lambda x : x) : return ''.join([wrap(f(x, depth+1, cDir)) for x in node.children])
    return g

class LexLine :
    """
    This class used to break up and represent a line (s) into
        matches?  (true or false if the pattern matches)
        glyph (what's the initial symbol on the front)
        keyword (which optionally breaks into)
            class
            id
        arglist
    """

    def keyword(self, s) :
        self.key = s
        if s.find(';') > 0 :
            parts = s.split(';')
            self.cls = parts[0]
            self.id = parts[1]
        else :
            self.cls = s
            self.id = ''
            
    def __init__(self,s,glyph) :
        self.args = []
        self.cls = ''
        self.id = ''
        self.key = ''
        self.matches = False
        self.s = s

        if s[0:len(glyph)] == glyph :
            self.matches = True
            s = s[len(glyph):].strip()
            if re.match('\S+\s+\S',s) :
                #has keyword and arg
                parts = s.split(' ')
                self.keyword(parts[0].strip())
                p = ' '.join(parts[1:])
                if p.find(',,') >= 0:
                    self.args = [x.strip() for x in p.split(',,')]
                else :
                    self.args = [p.strip()]

            else :
                self.keyword(s.strip())
                
class LexPageName :
    """
    Here's we're extracting information defining the name of a new page, 
    with optional blockCalls and rename
    Main format :
    &pageName [blockCall] [rename]
    eg.
    &myPage
    which will produce a page called myPage.html
    eg. 2
    &myPage :tpl > main.php
    which will produce a page wrapped in a calls the block tpl and saved to a file called main.php
    This class just cuts up the line into the requisite parts
    """
    def outFile(self, s) :
        m = re.search('>\s*(\S+)',s) 
        if m :
            self.outFileName = m.groups(0)[0]
            i = s.find('>')
            return s[:i].strip()
        else :
            return s.strip()
            
    def __init__(self, s, ext = 'html') :
        self.s = s
        self.matches = False
        self.pageName = ''
        self.outFileName = ''
        self.symbol = ''

        if re.match('&',s) :
            s = s[1:]
            self.matches = True
            s = self.outFile(s)

            if s.find(':') > 0 :
                parts = s.split(':')
                self.pageName = parts[0].strip()
                self.symbol = parts[1].strip()
                               
            else :
                self.pageName = s

            if self.outFileName == '' :
                self.outFileName = self.pageName + '.' + ext
